import base64
import csv
import json
import time
from urllib.request import Request, ProxyHandler, build_opener

SFDE_USERNAME = '<username>'
SFDE_PASSWORD = '<password>'
PROXIES = {'http': 'http://<username>:<password>@rb-proxy-de.bosch.com:8080',
           'https': 'https://<username>:<password>@rb-proxy-de.bosch.com:8080'}
sfdeProject = 'demo'

def basics_auth(username, password):
    """ Method which return a base 64 basic authentication
    :param username: your SFDE username
    :param password: your password username
    :return: returns string
    """
    credential = username + ':' + password
    encoded_credential = credential.encode('ascii')
    b_encoded_credential = base64.b64encode(encoded_credential)
    b_encoded_credential = b_encoded_credential.decode('ascii')
    b_auth = b_encoded_credential
    return 'Basic %s' % b_auth


def make_request(server_url, method, post_data=None):
    """ Method which handel a request and return response as json
    :param server_url:  the url for the request
    :param method: GET or POST method
    :param post_data: post data for Post method
    :return: returns json
    """
    proxy = ProxyHandler(PROXIES)
    opener = build_opener(proxy)
    r = Request(server_url)

    sfde_b_auth = basics_auth(SFDE_USERNAME, SFDE_PASSWORD)
    r.add_header('Authorization', sfde_b_auth)
    r.add_header('Content-Type', 'application/json')
    r.add_header('Accept', 'application/json')
    if method == 'POST':
        r_data = post_data
        r.data = r_data
        r.get_method = lambda: 'POST'

        handle = opener.open(r)
        content = handle.read().decode('utf8')
        response = json.loads(content)
        return response

    elif method == 'GET':
        r.get_method = lambda: 'GET'
        handle = opener.open(r)
        content = handle.read().decode('utf8')
        response = json.loads(content)
        return response
    else:
        print('Method %s is available' % method)


def main():
    try:
        # Server and the api service details
        server = 'https://bosch-iot-insights.com'
        serviceBaseUrl = '/mongodb-query-service/v2/' + sfdeProject
        service = '/submit-aggregation-query'
        query = '''
            {
                "collection": "demo_processed_data",
                "query":  [
                  {"$limit":10}
                ]
            }
        '''

        url = server + serviceBaseUrl + service  # construction of the final url for the first POST request
        data = query.encode("ascii")  # encoding format is ascii for the initial POST request

        res = make_request(url, 'POST', data)
        post_id = res['queryId']  # retreive the ID of the POST request
        print('post id', post_id)

        # Construct the GET request for POLLING the server using the POST ID
        get_url = server + serviceBaseUrl + "/queries" + "/" + post_id
        print("get_url: ", get_url)
        res_temp = make_request(get_url, 'GET')
        status_temp = res_temp['status']
        print('current status', status_temp)

        # POLLING the server in this loop till the status of the GET request is SUCCESSFUL
        while True:
            print('Entered inside loop')
            res_temp = make_request(get_url, 'GET')
            status_temp = res_temp['status']
            if status_temp == 'SUCCESSFUL':
                break
            print('current status - INSIDE LOOP', status_temp)
            time.sleep(2)

        # If the status is SUCCESSFUL,  we retrieve the results using the POST ID in the json format

        if status_temp == 'SUCCESSFUL':
            get_url_result = server + serviceBaseUrl + "/queries" + "/" + post_id + \
                             "/result?format=application%2Fjson"
            print("get_url_result: ", get_url_result)
            res_final = make_request(get_url_result, 'GET')

            # writing the JSON response to CSV file
            with open("vin_ccu_brake_events.csv", "w", newline='') as file:
                output = csv.writer(file)
                output.writerow(res_final[0].keys())  # header row
                for row in res_final:
                    output.writerow(row.values())
            print("-----------------------------final response--------------------------------")
            print(res_final)
        else:
            print("Error in data processing on the server: ", status_temp)
    except IOError as e:
        print("Problem with the request..")
        print(e)
        print(e.read())


if __name__ == '__main__': main()