import base64
import json
from urllib.request import Request, ProxyHandler, build_opener

SFDE_USERNAME = '<username>'
SFDE_PASSWORD = '<password>'
PROXIES = {'http': 'http://<username>:<password>@rb-proxy-de.bosch.com:8080',
           'https': 'https://<username>:<password>@rb-proxy-de.bosch.com:8080'}
sfdeProject = 'demo'


def basics_auth(username, password):
    """ Method which return a base 64 basic authentication
    :param username: your SFDE username
    :param password: your password username
    :return: returns string
    """
    credential = username + ':' + password
    encoded_credential = credential.encode('ascii')
    b_encoded_credential = base64.b64encode(encoded_credential)
    b_encoded_credential = b_encoded_credential.decode('ascii')
    b_auth = b_encoded_credential
    return 'Basic %s' % b_auth


def make_request(server_url, method, post_data=None):
    """ Method which handel a request and return response as json
    :param server_url:  the url for the request
    :param method: GET or POST method
    :param post_data: post data for Post method
    :return: returns json
    """
    proxy = ProxyHandler(PROXIES)
    opener = build_opener(proxy)
    r = Request(server_url)

    sfde_b_auth = basics_auth(SFDE_USERNAME, SFDE_PASSWORD)
    r.add_header('Authorization', sfde_b_auth)
    r.add_header('Content-Type', 'application/json')
    r.add_header('Accept', 'application/json')
    
    r_data = post_data
    r.data = r_data
    
    handle = opener.open(r)
    content = handle.read().decode('utf8')
    response = json.loads(content)
    return response


def main():
    try:
        # Server and the api service details
        server = 'https://bosch-iot-insights.com'
        serviceBaseUrl = '/mongodb-query-service/v2/' + sfdeProject
        service = '/execute-aggregation-query'
        query = '''
            {
                "collection": "demo_processed_data",
                "query":  [
                  {"$limit":1}
                ]
            }
        '''

        url = server + serviceBaseUrl + service  # construction of the final url for the first POST request
        data = query.encode("ascii")  # encoding format is ascii for the initial POST request
        res = make_request(url, 'POST', data)

        print(res)

    except IOError as e:
        print("Problem with the request..")
        print(e)
        

if __name__ == '__main__': main()